--[[
# Copyright 2020 Quarq
# This file is part of A-Bomb: Azeroth Bombardier.
# A-Bomb: Azeroth Bombardier is distributed under a BSD License.
# It is provided AS-IS and all warranties, express or implied, including, but not
# limited to, the implied warranties of merchantability or fitness for a particular
# purpose, are disclaimed.  See the LICENSE file for full information.
--]]
local MYNAME, _ = ...
MYNAME = GetAddOnMetadata(MYNAME, "X-ShortName") or MYNAME
local Addon = _G[MYNAME]
assert(Addon)

local sprintf = string.format
local floor = math.floor


local function random_color()
	return Addon.COLORS[ math.random(1, #Addon.COLORS) ].name
end

local function random_color_not(name)
	while (true) do
		local color = Addon.COLORS[ math.random(1, #Addon.COLORS) ]
		if not (color.name == name) then
			return color.name
		end
	end
end

local function random_cool()
	local colors = {"Green","Blue","Yellow"}
	return colors[ math.random(1,3) ]
end

local function random_warm()
	local colors = {"Red","Orange","Purple"}
	return colors[ math.random(1,3) ]
end


local NUISANCE, TROUBLEMAKER, BOMBARDIER, SKYMARSHALL = 1, 2, 3, 4

Addon.NUISANCE = NUISANCE
Addon.TROUBLEMAKER = TROUBLEMAKER
Addon.BOMBARDIER = BOMBARDIER
Addon.SKYMARSHALL = SKYMARSHALL
Addon.MAX_RANK = SKYMARSHALL

--[[
easiest
	kills
	kills, color
	kills, exclude
	kills, color, exclude
	kills, bombs
	kills, bombs, color
	kills, bombs, exclude
	kills, bombs, color, exclude
	kills, survivors, color
	kills, survivors
hardest
--]]

Addon.RANKS = {
	{
		level = NUISANCE,
		name = "Nuisance",
		missions = {
					{ kills=50 },
					{ kills=15, color="Red" },
					{ kills=15, color="Orange" },
					{ kills=15, color="Yellow" },
					{ kills=15, color="Green" },
					{ kills=15, color="Blue" },
					{ kills=15, color="Purple" },
				   },
		promotion = { missions=10, kills=1000, challenge={ kills=50, bombs=25, color=random_color() } }
	},
	{
		level = TROUBLEMAKER,
		name = "Troublemaker",
		missions = {
					{ kills=30, bombs=15, color="Red" },
					{ kills=30, bombs=15, color="Orange" },
					{ kills=30, bombs=15, color="Yellow" },
					{ kills=30, color=random_cool(),	exclude=random_warm() },
					{ kills=30, color=random_cool(),	exclude=random_warm() },
					{ kills=30, color=random_cool(),	exclude=random_warm() },
				   },
		promotion = { missions=50, kills=6000, challenge={ kills=100, bombs=50, color=random_warm(), exclude=random_cool() } }
	},
	{
		level = BOMBARDIER,
		name = "Bombardier",
		missions = {
					{ kills=200, bombs=80 },	-- 2.5 per
					{ kills=200, bombs=80 },	-- 2.5 per
					{ kills=150, bombs=50 },	-- 3.0 per
					{ kills=150, bombs=50 },	-- 3.0 per
					{ kills=150, bombs=50 },	-- 3.0 per
					{ kills=100, bombs=25 },	-- 4.0 per
					{ kills=100, bombs=25 },	-- 4.0 per
					{ kills=200, bombs=100, exclude=random_color() },
					{ kills=50, color="Red",	exclude=random_color_not("Red") },
					{ kills=50, color="Orange",	exclude=random_color_not("Orange") },
					{ kills=50, color="Yellow",	exclude=random_color_not("Yellow") },
					{ kills=50, color="Green",	exclude=random_color_not("Green") },
					{ kills=50, color="Blue",	exclude=random_color_not("Blue") },
					{ kills=50, color="Purple",	exclude=random_color_not("Purple") },
					{ kills=50, color="Red",	survivors=0.25 },
					{ kills=50, color="Orange",	survivors=0.25 },
					{ kills=50, color="Yellow",	survivors=0.25 },
					{ kills=50, color="Green",	survivors=0.25 },
					{ kills=50, color="Blue",	survivors=0.25 },
					{ kills=50, color="Purple",	survivors=0.25 },
					{ kills=100, survivors=0.50 },
				   },
		promotion = { missions=100, kills=15000, challenge={ kills=100, color=random_color(), survivors=0.25 } }
	},
	{
		level = SKYMARSHALL,
		name = "Sky Marshall"
	}
}


function Addon : InitRanks()
	Addon.DB.rank = Addon.DB.rank or NUISANCE
	Addon.DB.total_kills = Addon.DB.total_kills or 0
	Addon.DB.total_missions = Addon.DB.total_missions or 0	-- completed
	Addon.DB.total_sorties = Addon.DB.total_sorties or 0	-- flights, regardless of mission/outcome
end


function Addon : GetCurrentRank()
	return Addon.RANKS[Addon.DB.rank or NUISANCE]
end


function Addon : GetNextRank()
	local current = self:GetCurrentRank()
	return Addon.RANKS[current.level + 1]
end


function Addon : GetMission()
	local rank = self:GetCurrentRank()
	if (rank and rank.missions) then
		-- if no completed mission, only offer the simplest
		if (Addon.DB.total_missions == 0) then
			return rank.missions[1], false
		else -- he has completed at least one already
			-- if he's completed this rank's mission & kills objectives, then it's time for the challenge mission
			if (Addon.DB.total_kills >= rank.promotion.kills and Addon.DB.total_missions >= rank.promotion.missions) then
				rank.promotion.challenge.title = self:GetNextRank().name .. " Challenge"
				return rank.promotion.challenge, false
			else
				local which = math.random(1, #rank.missions)
				local mission = rank.missions[which]

				-- local letters = "ABCDEFGHJKLMNPQRSTUVWXYZ"
				-- local which = math.random(1, letters:len())
				-- mission.title = sprintf("%s-%d/%s", rank.name:sub(1,1), math.random(10,99), letters:sub(which,which))

				return mission, true
			end
		end
	else
		return nil
	end
end
