--[[
# Copyright 2020 Quarq
# This file is part of A-Bomb: Azeroth Bombardier.
# A-Bomb: Azeroth Bombardier is distributed under a BSD License.
# It is provided AS-IS and all warranties, express or implied, including, but not
# limited to, the implied warranties of merchantability or fitness for a particular
# purpose, are disclaimed.  See the LICENSE file for full information.
--]]
local MYNAME, _ = ...
MYNAME = GetAddOnMetadata(MYNAME, "X-ShortName") or MYNAME
local Addon = _G[MYNAME]
assert(Addon)

local TIMER = CreateFrame("Frame")
TIMER.resolution = 0.01


Addon.TIMER = TIMER


TIMER.timers = {}
TIMER.count = 0
TIMER.elapsed = 0
TIMER.locked = false
TIMER.tick = 0


local _foo = 0

function TIMER : Start(duration, progress, completion)
	assert(not self.locked)

	if (duration <= 0) then return end

	local timer = {}
	timer.time = self.elapsed + duration	-- current time plus duration
	timer.duration = duration -- only for debugging
	timer.progress = progress
	timer.completion = completion

	-- debugging
	_foo = _foo + 1
	timer.name = "TIMER" .. _foo

	local index = nil
	for i, other in ipairs(self.timers) do
		if (timer.time < other.time) then
			index = i
			break
		end
	end

	if (index) then
		table.insert(self.timers, index, timer)
	else
		table.insert(self.timers, timer)
	end
	self.count = self.count + 1

	if (self.debug) then print(string.format("%6.2d: %s", self.elapsed, timer.name), "started", (timer.time-self.elapsed) .. "s") end

	self:SetScript("OnUpdate", self.OnUpdate)

	return timer
end


function TIMER : Cancel(timer)
	assert(not self.locked)

	if (timer == nil) then return end

	for i, t in ipairs(self.timers) do
		if (t == timer) then
			table.remove(self.timers, i)
			self.count = self.count - 1

			if (self.count == 0) then
				self:SetScript("OnUpdate", nil)
			end

			if (self.debug and timer.time > self.elapsed) then print(string.format("%6.2d: %s", self.elapsed, timer.name), "cancelled", (timer.time-self.elapsed) .. "s") end

			return
		end
	end
end


function TIMER : OnUpdate(elapsed)
	-- should not get called unless there's at least one timer
	assert(self.count > 0)
	assert(self.count == #self.timers)

	self.elapsed = self.elapsed + elapsed

	-- process any that have finished
	while (self.count > 0) do
		local timer = self.timers[1]
		if (self.elapsed >= timer.time) then
			if (self.debug) then print(string.format("%6.2d: %s", self.elapsed, timer.name), "finished") end
			if (timer.completion) then timer.completion(timer) end
			-- IMPORTANT:  completion handler may modify my state - make no assumptions
			self:Cancel(timer) -- harmless if already cancelled
		else
			-- no more expired timers
			break
		end
	end

	-- call progress handlers, subject to resolution requirement
	self.tick = self.tick + elapsed
	if (self.tick >= self.resolution) then
		self.tick = self.tick - self.resolution -- basically, tick = tick % resolution

		self.locked = true
		for i, timer in ipairs(self.timers) do
			timer.elapsed = timer.duration - (timer.time - self.elapsed) --self.elapsed - timer.time - timer.duration
			if (timer.progress) then timer.progress(timer) end
		end
		self.locked = false
	end

end


function TIMER : Dump()
	for i, timer in ipairs(self.timers) do
		print(string.format("%2d %4.2f/%4.2f %s", i, timer.time-self.elapsed, timer.duration, timer.name))
	end
end
