--[[
# Copyright 2020 Quarq
# This file is part of A-Bomb: Azeroth Bombardier.
# A-Bomb: Azeroth Bombardier is distributed under a BSD License.
# It is provided AS-IS and all warranties, express or implied, including, but not
# limited to, the implied warranties of merchantability or fitness for a particular
# purpose, are disclaimed.  See the LICENSE file for full information.
--]]
local MYNAME, _ = ...
MYNAME = GetAddOnMetadata(MYNAME, "X-ShortName") or MYNAME
local Addon = _G[MYNAME]
assert(Addon)


string.trim = function(s) return msg:match("^%s*(.-)%s*$") end


-- quest colors
Addon.red		= function (text) return text and ("|cffff1a1a" .. text .. "|r") end
Addon.orange	= function (text) return text and ("|cffff8040" .. text .. "|r") end
Addon.yellow	= function (text) return text and ("|cffffff00" .. text .. "|r") end
Addon.blue		= function (text) return text and ("|cff0000c0" .. text .. "|r") end
Addon.gold		= function (text) return text and ("|cffffd000" .. text .. "|r") end
Addon.green		= function (text) return text and ("|cff40c040" .. text .. "|r") end
Addon.gray		= function (text) return text and ("|cffb0b0b0" .. text .. "|r") end
Addon.gray80	= function (text) return text and ("|cffcccccc" .. text .. "|r") end

Addon.purple	= function (text) return text and ("|cffc060c0" .. text .. "|r") end	-- like the floating XP notifications
Addon.white		= function (text) return text and ("|cffffffff" .. text .. "|r") end
Addon.almost_white = function (text) return text and ("|cffe8e8e8" .. text .. "|r") end

Addon.tooltip_white		= function (text) return text and ("|cffe8e8e8" .. text .. "|r") end
Addon.tooltip_gray		= function (text) return text and ("|cff909090" .. text .. "|r") end
Addon.tooltip_gray80	= function (text) return text and ("|cffd0d0d0" .. text .. "|r") end

Addon.color_friendly	= function (text) return text and ("|cff00ff00" .. text .. "|r") end

Addon.difficulty_color = function(level, text)
	if (text == nil) then return nil end
    local diff = level - UnitLevel("player")
    local color = diff>=5 and Addon.red or diff>=3 and Addon.orange or diff>=-2 and Addon.yellow or diff>=0-GetQuestGreenRange() and Addon.green or Addon.gray
    return color(text)
end

Addon.checkmark = function (size)
	size = 0 + size
	local offset = -math.floor(10*size/9+0.5)/10
	-- return "|TInterface/Buttons/UI-CheckBox-Check:"..size..":"..size..":0:"..offset.."|t"
	return "|TInterface/Buttons/UI-CheckBox-Check:"..(size*22/32)..":"..size..":0:"..offset..":32:32:0:32:4:26|t"
-- |Tpath:height[:width[:offsetX:offsetY:[textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel[:rVertexColor:gVertexColor:bVertexColor]]]]|t
end

Addon.questionmark = function (size, bGray)
	size = 1 + size
	local dim = 32
	local w = dim - 2 * 5	-- texture width in pixels, minus the same margin from both side
	w = w - w%2	-- even width
	local left, right = dim/2-w/2, dim/2+w/2
	local top, bottom = 1, 32
	local dx, dy = 0, -1
	local file = "Interface/Addons/Questie-Quarq/maps/node_complete" .. (bGray and "_gray" or "")
	return "|T"..file..":"..size..":"..(size*w/dim)..":"..dx..":"..dy..":"..dim..":"..dim..":"..left..":"..right..":"..top..":"..bottom.."|t"
	-- |Tpath:height[:width[:offsetX:offsetY:[textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel[:rVertexColor:gVertexColor:bVertexColor]]]]|t
end

Addon.exclamationmark = function (size)
	size = 0 + size
	local dim = 32
	local w = dim - 2 * 5	-- texture width in pixels, minus the same margin from both side
	w = w - w%2	-- even width
	local left, right = dim/2-w/2, dim/2+w/2
	local top, bottom = 1, 32
	local dx, dy = 0, -2
	return "|TInterface/Addons/Questie-Quarq/maps/node_available:"..size..":"..(size*w/dim)..":"..dx..":"..dy..":"..dim..":"..dim..":"..left..":"..right..":"..top..":"..bottom.."|t"
	-- |Tpath:height[:width[:offsetX:offsetY:[textureWidth:textureHeight:leftTexel:rightTexel:topTexel:bottomTexel[:rVertexColor:gVertexColor:bVertexColor]]]]|t
end

-- -- item quality codes
-- Addon.Q_GRAY	= 0
-- Addon.Q_WHITE	= 1
-- Addon.Q_GREEN	= 2
-- Addon.Q_BLUE	= 3
-- Addon.Q_PURPLE	= 4
-- -- and there are others



Addon.goldformat = function (amount)
	if (amount < 100) then
		return string.format("0.%02d", amount)	-- 0.09
	elseif (amount < 100 * 100) then
		return string.format("%d.%02d", math.floor(amount/100)%100, amount%100)	-- 1.09
	else
		return string.format("%d.%02d.%02d", Addon.commaformat(math.floor(amount/100/100)), math.floor(amount/100)%100, amount%100)	-- 1.09.75
	end
end


Addon.commaformat = function(amount)
	local formatted = ""..amount
	while true do
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
		if ( k == 0) then
			break
		end
	end
	return formatted
end


-- GetCoinTextureString() doesn't show zero silver and/or copper parts, so I do this
local GoldCoin = GetCoinTextureString(100*100, 10):sub(2)
local SilverCoin = GetCoinTextureString(100, 10):sub(2)
local CopperCoin = GetCoinTextureString(1, 10):sub(2)

Addon.coinformat = function(amount)
	local gg = math.floor(amount/100/100)
	local ss = math.floor(amount/100) % 100
	local cc = amount % 100
	-- -- this one always shows 2-digit silver and 2-digit copper, even "00"s
	-- if (gg > 0) then
	-- 	return string.format("%d%s %02d%s %02d%s", Addon.commaformat(gg), GoldCoin, ss, SilverCoin, cc, CopperCoin)
	-- elseif (ss > 0) then
	-- 	return string.format("%d%s %02d%s", ss, SilverCoin, cc, CopperCoin)
	-- else
	-- 	return string.format("%d%s", cc, CopperCoin)
	-- end

	-- this one only shows non-00 parts... so, 3g 08s, or 3g 99c are possibilities
	local text = ""
	if (cc > 0) then
		text = string.format(gg==0 and ss==0 and "%d%s" or "%02d%s", sccs, CopperCoin) .. " " .. text
	end
	if (ss > 0) then
		text = string.format(gg==0 and "%d%s" or "%02d%s", ss, SilverCoin) .. " " .. text
	end
	if (gg > 0) then
		text = Addon.commaformat(gg) .. GoldCoin .. " " .. text
	end
	return text:sub(1,-2) 	-- all but trailing space
end


Addon.dump = function(t, title)
	if (title) then print(title) end
	for k, v in pairs(t) do
		local t = type(v)
		print("   ", k, type(v), (t=="string" and ("= \""..v.."\"") or t=="number" and ("= "..v) or ""))
	end
end


Addon.hsv = function(h,s,v)
	-- assert(h and s and v)

	-- clamp sat and value
	s = s<0 and 0 or s>1 and 1 or s
	v = v<0 and 0 or v>1 and 1 or v

	-- 0 sat means grayscale
	if (s == 0) then
		return v,v,v
	end

	h = h<0 and -h or h
	h = h - math.floor(h)	-- just the fractional part

	local i = math.floor(h*6)
    local f = h*6 - i
	-- assert(f > -1.0 and f < 1.0)

    local p = v * (1 - s);
    local q = v * (1 - s*f);
    local t = v * (1 - s*(1-f));

	if (i == 0) then
		-- assert(p>=0 and p<=1)
		-- assert(t>=0 and t<=1)
		return v,t,p
	elseif (i == 1) then
		-- assert(p>=0 and p<=1)
		-- assert(q>=0 and q<=1)
		return q,v,p
	elseif (i == 2) then
		-- assert(p>=0 and p<=1)
		-- assert(t>=0 and t<=1)
		return p,v,t
	elseif (i == 3) then
		-- assert(p>=0 and p<=1)
		-- assert(q>=0 and q<=1)
		return p,q,v
	elseif (i == 4) then
		-- assert(p>=0 and p<=1)
		-- assert(t>=0 and t<=1)
		return t,p,v
	elseif (i == 5) then
		-- assert(p>=0 and p<=1)
		-- assert(q>=0 and q<=1)
		return v,p,q
	elseif (i == 6) then
		-- assert(p>=0 and p<=1)
		-- assert(q>=0 and q<=1)
		return v,p,q
	else
		assert(false)
		return 0,0,0
	end
end


Addon.hsva = function(h,s,v,a)
	local h,s,v = Addon.hsv(h,s,v)
	return h,s,v,a
end


Addon.mapdistance2 = function(ax,ay, bx,by)
	local dx,dy = ax-bx, (ay-by)/1.5
	return dx*dx + dy*dy
end


Addon.get_mouse_position_map = function ()
    local x,y = GetCursorPosition();

    local scale = WorldMapFrame:GetCanvas():GetEffectiveScale();
    x = x / scale;
    y = y / scale;

    local left = WorldMapFrame:GetCanvas():GetLeft();
    local top = WorldMapFrame:GetCanvas():GetTop();
    local width = WorldMapFrame:GetCanvas():GetWidth();
    local height = WorldMapFrame:GetCanvas():GetHeight();

    x = (x - left) / width;
    y = (top - y) / height;

    return x,y
end



local function add_line(tip, fontsize, left, right)
    if (right) then
        tip:AddDoubleLine(left, right)
    else
        tip:AddLine(left)
    end
    tip.nlines = tip.nlines + 1

    local prefix = tip:GetName() .. "Text"
    _G[prefix.."Left"..tip.nlines] : SetFont("Fonts\\FRIZQT__.TTF", fontsize)
    _G[prefix.."Right"..tip.nlines] : SetFont("Fonts\\FRIZQT__.TTF", fontsize)
end

Addon.TooltipMixin = {}
Addon.TooltipMixin.clear_lines		= function(tip, left, right) tip:ClearLines(); tip.nlines=0 end
Addon.TooltipMixin.add_title_line	= function(tip, left, right) add_line(tip, 13, left, right) end
Addon.TooltipMixin.add_detail_line	= function(tip, left, right) add_line(tip, 12, left, right) end
Addon.TooltipMixin.add_small_line	= function(tip, left, right) add_line(tip, 11, left, right) end
